// main.cpp
//
// ICS 65 Fall 2012
// Code Example
//
// A quick demonstration of using our Vector template class in a few ways.

#include <iostream>
#include <string>
#include "Vector.h"



// This template function allows me to print the values stored in any
// vector, no matter its type.  Note that I could have used standard
// library functions like std::for_each or a range-based "for" loop
// instead, but our Vector template class doesn't support iterators
// the way the standard library containers do.  If you want something
// interesting to work on, add that functionality to our Vector class,
// so that it becomes possible to say this:
//
//     Vector<int> v;
//     ...
//     for (int i : v)
//     {
//         std::cout << i << std::endl;
//     }

template <typename T>
void printAll(const Vector<T>& v)
{
	for (unsigned int i = 0; i < v.size(); i++)
	{
		std::cout << v.at(i) << std::endl;
	}
}


void creatingACoupleOfVectors()
{
	std::cout << "creatingACoupleOfVectors" << std::endl;

	Vector<int> v1;
	v1.add(3);
	v1.add(2);
	v1.add(5);
	v1.add(8);
	v1.add(7);
	v1.add(2);
	v1.add(9);
	v1.add(6);
	v1.add(1);
	printAll(v1);

	Vector<std::string> v2;
	v2.add("Alex");
	v2.add("is");
	v2.add("happy");
	v2.add("because");
	v2.add("Boo");
	v2.add("is");
	v2.add("nearby");
	printAll(v2);
}



// This function has some code commented out that will trigger compile-time
// errors related to the constraints on the type T in Vector<T>.  It makes
// use of a class Uncopyable, whose objects cannot be copied.

class Uncopyable
{
public:
	Uncopyable() { }

private:
	Uncopyable(const Uncopyable&);
	Uncopyable& operator=(const Uncopyable&);
};


class NoDefaultConstructor
{
public:
	NoDefaultConstructor(int i, int j) { }
};


void triggeringCompileTimeErrors()
{
	// Notice that this is fine.  That's because the constructor doesn't
	// impose the constraint that T's be copyable.
	Vector<Uncopyable> v;

	// These, on the other hand, would be problematic, because all of
	// these functions do impose that constraint:
//	Vector<Uncopyable> v2 = v;
//	v = v2;
//	v.add(Uncopyable());

	// This, too, is problematic, because NoDefaultConstructor objects have
	// no default constructor, which is required in order to be able to
	// construct a Vector.
//	Vector<NoDefaultConstructor> v3;
}


int main()
{
	creatingACoupleOfVectors();

	return 0;
}
